/**
* @author Dirk Nordmann
* @date   14.01.2002
*/
package xml2rtf.rtf;

import util.*;
import xml2rtf.templates.*;
import java.io.*;
import java.util.Enumeration;
import java.util.Vector;

public class RtfWriter
{
	
  /**
  * Constructor
  * @params   filename		Filename and -path of the RTF-File to be generated.
  * @returns  RtfWriter		New instance of RtfWriter
  */
  public RtfWriter(String filename)
  {
    _list = new Vector(10);
    _filename = filename;
    _tmpfilename = FilePathUtility.getDirectory(filename) + File.separator +  "~temp.rtf";
    System.out.println("[RtfWriter.constructor]: Writing to temporary file " + _tmpfilename );
    try {
      outputStream = new BufferedWriter(new FileWriter(_tmpfilename));
    }
    catch (IOException _ex) {
      System.out.println("*****[RtfWriter.constructor]: FEHLER -- RTF-Datei " + _tmpfilename + " kann nicht geffnet werden. *****");
      System.exit(1);
    }
  }

  /**
  * Write to RTF-file.
  * @params s		Text to write.
  */
  public void write(String s)
  {
    try {
      outputStream.write(s);
    }
    catch(IOException _ex) {
      System.out.println("*****[RtfWriter.write]: FEHLER -- RTF-Datei " + _filename + " kann nicht geschrieben werden. *****");
      System.exit(1);
    }
  }

  /**
  * Close temporary file and write final RTF-file.
  */
  public void close()
  {
    if (outputStream != null) {
    	try {
        //* Close temporary file *
        outputStream.close();
        outputStream = null;
                
        //* Open final RTF-file *
        // (use BufferedWriter for 7-bit ASCII-files)
        BufferedWriter rtf = new BufferedWriter(new FileWriter(_filename));
                
        //*****  HEADER  *****
        //---  header 
   		  rtf.write(STD_HEADER);
       	//---  fonttbl : Table of Fonts
	      rtf.write(STD_FONTS);
   		  //---  colortbl : Table of Colors
   		  printColortbl(rtf);
   		  //---  stylesheet
       	rtf.write(STD_STYLES);
	      //---  listtable : Table of Lists and Enumerations
	      printListtable(rtf);
       	//---  margins
   		  rtf.write( "\n\\paperw" + ConversionTemplate.paperWidthInTwips
   	  		+ "\\paperh" + ConversionTemplate.paperHeightInTwips
   	      + "\\margl" + ConversionTemplate.leftMarginInTwips 
					+ "\\margr" + ConversionTemplate.rightMarginInTwips 
					+ "\\margt" + ConversionTemplate.topMarginInTwips 
					+ "\\margb" + ConversionTemplate.bottomMarginInTwips 
					+ "\n");
        //--- ??? (don't know yet what this is good for) 
        rtf.write(STD_UNKNOWN);
    		        
        //*****  BODY  *****
        // Write body from temporary file
        // (use BufferedReader for 7-bit ASCII-files)
        rtf.write(DEFAULT);
        BufferedReader tmp = new BufferedReader(new FileReader(_tmpfilename));
        char ac[] = new char[1024];
        int j;
        while((j = tmp.read(ac, 0, ac.length)) != -1) 
          rtf.write(ac, 0, j);
        tmp.close();
        // Delete temporary file
        (new File(_tmpfilename)).delete();
                
        //*****  FOOTER  *****
        rtf.write(ENDGROUP);
                               
        // Close final RTF-file
        rtf.close();
      }
      catch (IOException _ex) {
        System.out.println("*****[RtfWriter.close]: IO-AUSNAHMEFEHLER -- While generating final RTF file: " + _ex + ". *****");
      }
    } 
  }


/* ----------------------------------------------------------------------

		L I S T S
		
   ----------------------------------------------------------------------
 */
     
/* ################################################################################
   ###  getListNumber
   ###
   ###	Creates new List-ID  and stores it for later processing of listtable.
   ###
   ###  IN:  list_type		Type of List (UL=unordered, OL=ordered)
   ###  OUT: int			List-ID
   ################################################################################
 */
public int getListNumber(int list_type)
{
	int list_number = _list.size();
    Integer iListNumber = new Integer(list_type);
    _list.addElement(iListNumber);
	return list_number + LIST_OFFSET;
}

/* ################################################################################
   ###  printListItemStart
   ###
   ###	Prints RTF-startsequence for list-items.
   ###
   ###  IN:  list_type		Type of List (UL=unordered, OL=ordered)
   ###		 list_number	List-ID
   ###  OUT: None
   ################################################################################
 */
public void printListItemStart(int list_type, int list_number)
{
	String listitem_start = "";
    		
	switch(list_type)
    {
	case ConversionTemplate.LIST_OL: 
		listitem_start = LISTITEM_START_OL;
		break;
	case ConversionTemplate.LIST_UL: 
		listitem_start = LISTITEM_START_UL;
		break;
    }
        
	write( LISTITEM_START_BEFORE );
	write( StringUtil.replace(listitem_start, "%LIST_NUMBER", list_number + ""));
	write( LISTITEM_START_AFTER );
}

/* ################################################################################
   ###  printListtable
   ###
   ###	Prints RTF-headersequence for \\listtable and 
   ###  \\listoverridetable.
   ###
   ###  IN: 	out			Outputstream
   ###  OUT:	None
   ################################################################################
 */
private void printListtable(BufferedWriter out)
  	throws IOException
{
	//*****  \\listtable  *****
	out.write(LISTTABLE_START);
   	int i = 0;
	for (Enumeration enumeration = _list.elements(); enumeration.hasMoreElements();)
    {
   		String listtableentry = LISTTABLEENTRY_START;
        switch (((Integer)enumeration.nextElement()).intValue())
        {
		case ConversionTemplate.LIST_UL:
			listtableentry += LISTTABLEENTRY_UL;
		  	break;
   		case ConversionTemplate.LIST_OL:
			listtableentry += LISTTABLEENTRY_OL;
			break;
  		}
		listtableentry += LISTTABLEENTRY_END;
		listtableentry = StringUtil.replace(listtableentry, "%LISTTEMPLATEID", (LIST_TEMPLATEID_OFFSET + i ) + "");
		listtableentry = StringUtil.replace(listtableentry, "%LISTID", (LIST_SEED + i) + "" );
		out.write(listtableentry);
        i++;
    }
   	out.write(ENDGROUP);
   		
   	//*****  \\listoverridetable  *****
   	out.write(LISTOVERRIDETABLE_START);
   	i = 0;
	for (Enumeration enumeration = _list.elements(); enumeration.hasMoreElements();)
    {
   		int list_type = ((Integer)enumeration.nextElement()).intValue();
   		String listoverridetable_entry = LISTOVERRIDETABLE_ENTRY;
   		listoverridetable_entry = StringUtil.replace(listoverridetable_entry, "%LISTID", (LIST_SEED + i) + "" );
   		listoverridetable_entry = StringUtil.replace(listoverridetable_entry, "%LS", (LIST_OFFSET + i) + "" );
        out.write(listoverridetable_entry);
        i++;
    }
	out.write(ENDGROUP);
}
    
    
    
    /* ----------------------------------------------------------------------
    
    		C O L O R S
    		
    	 ----------------------------------------------------------------------
     */
	private static Vector _colorlist = new Vector(10);
     
    public static String getColor(String controlword, int color)
    {
        return "\\" + controlword + addColor(color);
    }

    /**
    * @returns  String
    */
    public static String getColor(String color)
    {
      int color_code = ConversionTemplate.decodeColor(color);
   		return getColor("cf",color_code);
    }
    
    /**
    * @returns  String
    */
    public static String getBackgroundColor(String bgcolor)
    {
    	int color_code = ConversionTemplate.decodeColor(bgcolor);
    	return getColor("highlight",color_code);
    }
    
    private static int addColor(int color)
    {
        Integer iColor = new Integer(color);
        int j = _colorlist.indexOf(iColor);
        if(j == -1)
        {
            _colorlist.addElement(iColor);
            j = _colorlist.size() - 1;
        }
        return j + COLOR_OFFSET;
    }

    /* ######################################################################
       ###  printColortbl
       ###
       ###	Prints RTF-headersequence for \\colortbl.
       ###
       ###  IN: 	out			Outputstream
       ###  OUT:	None
       ######################################################################
     */
    private void printColortbl(BufferedWriter out)
   		throws IOException
    {
  		//*****  \\colortbl  *****
        out.write(STD_COLORS_START);
        int i;
		    for (Enumeration enumeration = _colorlist.elements(); 
    		     enumeration.hasMoreElements(); 
        		 out.write("\n\\red" + ((i & 0xff0000) >> 16) + "\\green" + ((i & 0xff00) >> 8) + "\\blue" + (i & 0xff) + ";\n"))
           i = ((Integer)enumeration.nextElement()).intValue();
	    out.write(ENDGROUP);
	}
		
private static final int COLOR_OFFSET = 3;
private static final int LIST_OFFSET = 8;
private static final int LIST_TEMPLATEID_OFFSET = 67567617;
private static final int LIST_SEED = 19433432;

public static final String DEFAULT = "\\f1\\fs22";
public static final String PLAIN = "\\plain" + DEFAULT;
public static final String PARD = "\\pard" + DEFAULT;
public static final String EOL = "\\par\n";
public static final String LINE_BREAK = "\\line\n";
private BufferedWriter outputStream;
private String _filename;
private String _tmpfilename;
private Vector _list;
    
public static final String ENDGROUP = "}\n";
private static final String STD_HEADER
	= "{\\rtf1\\ansi\\ansicpg1252\\uc1 \\deff16\\deflang1033\\deflangfe1031\n";
private static final String STD_COLORS_START
  = "{\\colortbl " 
  + "\n\\red0\\green0\\blue0;"
  + "\n\\red0\\green0\\blue128;"
  + "\n\\red0\\green0\\blue255;\n";
private static final String STD_FONTS
  = "{\\fonttbl"
  + "\n{\\f0\\froman\\fcharset0\\fprq2{\\*\\panose 02020603050405020304}Times New Roman;}"
  + "\n{\\f1\\fswiss\\fcharset0\\fprq2{\\*\\panose 020b0604020202020204}Arial;}"
  + "\n{\\f2\\fmodern\\fcharset0\\fprq1{\\*\\panose 02070309020205020404}Courier New;}"
  + "\n{\\f3\\froman\\fcharset2\\fprq2{\\*\\panose 05050102010706020507}Symbol;}"
  + "\n{\\f14\\fnil\\fcharset2\\fprq2{\\*\\panose 05000000000000000000}Wingdings;}"
  + "\n{\\f34\\froman\\fcharset238\\fprq2 Times New Roman CE;}"
  + "\n{\\f35\\froman\\fcharset204\\fprq2 Times New Roman Cyr;}"
  + "\n{\\f37\\froman\\fcharset161\\fprq2 Times New Roman Greek;}"
  + "\n{\\f38\\froman\\fcharset162\\fprq2 Times New Roman Tur;}"
  + "\n{\\f39\\froman\\fcharset177\\fprq2 Times New Roman (Hebrew);}"
  + "\n{\\f40\\froman\\fcharset178\\fprq2 Times New Roman (Arabic);}"
  + "\n{\\f41\\froman\\fcharset186\\fprq2 Times New Roman Baltic;}"
  + "\n{\\f42\\fswiss\\fcharset238\\fprq2 Arial CE;}"
  + "\n{\\f43\\fswiss\\fcharset204\\fprq2 Arial Cyr;}"
  + "\n{\\f45\\fswiss\\fcharset161\\fprq2 Arial Greek;}"
  + "\n{\\f46\\fswiss\\fcharset162\\fprq2 Arial Tur;}"
  + "\n{\\f47\\fswiss\\fcharset177\\fprq2 Arial (Hebrew);}"
  + "\n{\\f48\\fswiss\\fcharset178\\fprq2 Arial (Arabic);}"
  + "\n{\\f49\\fswiss\\fcharset186\\fprq2 Arial Baltic;}"
  + "\n{\\f50\\fmodern\\fcharset238\\fprq1 Courier New CE;}"
  + "\n{\\f51\\fmodern\\fcharset204\\fprq1 Courier New Cyr;}"
  + "\n{\\f53\\fmodern\\fcharset161\\fprq1 Courier New Greek;}"
  + "\n{\\f54\\fmodern\\fcharset162\\fprq1 Courier New Tur;}"
  + "\n{\\f55\\fmodern\\fcharset177\\fprq1 Courier New (Hebrew);}"
  + "\n{\\f56\\fmodern\\fcharset178\\fprq1 Courier New (Arabic);}"
  + "\n{\\f57\\fmodern\\fcharset186\\fprq1 Courier New Baltic;}"
  + "\n}"
	;
private static final String STD_STYLES
	= "{\\stylesheet\n"
	+ "{\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\snext0 Normal;}\n"
	+ "{\\s1\\ql \\li0\\ri0\\sb360\\sl300\\slmult1 \\keepn\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\b\\f1\\fs34\\cf" + getColor("darkblue") + "\\lang1031\\langfe1031\\kerning32\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 heading 1;}\n"
	+ "{\\s2\\ql \\li0\\ri0\\sb240\\sa60\\sl300\\slmult1 \\keepn\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\b\\i\\f1\\fs28\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 heading 2;}\n"
	+ "{\\s3\\ql \\li0\\ri0\\sb240\\sa60\\sl300\\slmult1 \\keepn\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\b\\f1\\fs26\\cf" + getColor("darkblue") + "\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 heading 3;}\n"
	+ "{\\s4\\ql \\li0\\ri0\\sb240\\sa60\\sl300\\slmult1 \\keepn\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\b\\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 heading 4;}\n"
	+ "{\\s5\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 heading 5;}\n"
	+ "{\\s6\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 heading 6;}\n"
	+ "{\\s7\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 heading 7;}\n"
	+ "{\\s8\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 heading 8;}\n"
	+ "{\\s9\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 heading 9;}\n"
	+ "{\\*\\cs10 \\additive Default Paragraph Font;}\n"
	+ "{\\s15\\qc \\li0\\ri0\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs20\\cf" + getColor("darkblue") + "\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext15 Tabellenspaltentitel;}\n"
	+ "{\\s16\\ql \\li0\\ri0\\sb120\\sa240\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\i\\f1\\fs18\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 caption;}\n"
	+ "{\\s17\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\b\\f1\\fs20\\cf" + getColor("darkblue") + "\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 Tabellentitel;}{\\*\\cs18 \\additive \\super \\sbasedon10 Hochgestellt;}{\\*\\cs19 \\additive \\sub Tiefgestellt;}\n"
	+ "{\\s20\\ql \\li714\\ri714\\sb120\\sa120\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin714\\lin714\\itap0 \\b\\f1\\fs20\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 Merksatz;}\n"
	+ "{\\s21\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs20\\cf" + getColor("darkblue") + "\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext21 Tabellenzeilentitel;}\n"
	+ "{\\s22\\ql \\fi-360\\li360\\ri0\\sl300\\slmult1\\widctlpar\\jclisttab\\tx360\\aspalpha\\aspnum\\faauto\\ls22\\adjustright\\rin0\\lin360\\itap0 \\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext22 Referenz;}\n"
	+ "{\\s23\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\tqc\\tx4536\\tqr\\tx9072\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext23 header;}\n"
	+ "{\\s24\\ql \\li0\\ri0\\sl300\\slmult1\\widctlpar\\tqc\\tx4536\\tqr\\tx9072\\aspalpha\\aspnum\\faauto\\adjustright\\rin0\\lin0\\itap0 \\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext24 footer;}\n"
	+ "{\\s25\\ql \\fi-1701\\li1701\\ri0\\sl300\\slmult1\\widctlpar\\jclisttab\\tx1701\\aspalpha\\aspnum\\faauto\\ls29\\adjustright\\rin0\\lin1701\\itap0 \\f1\\fs40\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext26 MetaTitle;}\n"
	+ "{\\s26\\ql \\fi-1701\\li1701\\ri0\\sl300\\slmult1\\widctlpar\\jclisttab\\tx1701\\aspalpha\\aspnum\\faauto\\ls28\\adjustright\\rin0\\lin1701\\itap0 \\f1\\fs20\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext27 MetaTitleAlt;}\n"
	+ "{\\s27\\ql \\fi-1701\\li1701\\ri0\\sl300\\slmult1\\widctlpar\\jclisttab\\tx1701\\aspalpha\\aspnum\\faauto\\ls27\\adjustright\\rin0\\lin1701\\itap0 \\f1\\fs20\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext28 MetaDate;}\n"
	+ "{\\s28\\ql \\fi-1701\\li1701\\ri0\\sl300\\slmult1\\widctlpar\\jclisttab\\tx1701\\aspalpha\\aspnum\\faauto\\ls27\\ilvl1\\adjustright\\rin0\\lin1701\\itap0 \\f1\\fs20\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext29 MetaCreator;}\n"
	+ "{\\s29\\ql \\fi-1701\\li1701\\ri0\\sl300\\slmult1\\widctlpar\\jclisttab\\tx1701\\aspalpha\\aspnum\\faauto\\ls27\\ilvl3\\adjustright\\rin0\\lin1701\\itap0 \\f1\\fs20\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext0 MetaSubject;}\n"
	+ "{\\s30\\ql \\fi-1701\\li1701\\ri0\\sl300\\slmult1\\widctlpar\\brdrb\\brdrs\\brdrw10\\brsp20 \\jclisttab\\tx1701\\aspalpha\\aspnum\\faauto\\ls30\\adjustright\\rin0\\lin1701\\itap0 \\f1\\fs20\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext1 MetaSource;}\n"
	+ "{\\s31\\ql \\fi-1701\\li1701\\ri0\\sl300\\slmult1\\widctlpar\\jclisttab\\tx1701\\aspalpha\\aspnum\\faauto\\ls27\\ilvl5\\adjustright\\rin0\\lin1701\\itap0 \\f1\\fs20\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031 \\sbasedon0 \\snext30 MetaDescription;}\n"
	+ "}\n";

private final static String LISTITEM_START_BEFORE
	= "\n\\fi-360\\li720\\ri0\\sl300\\slmult1\\widctlpar\\jclisttab\\tx720\\aspalpha\\aspnum\\faauto\\ls31\\adjustright\\rin0\\lin720\\itap0 ";
private final static String LISTITEM_START_UL
	= "\n{\\*\\pn \\pnlvlblt\\ilvl0\\ls%LIST_NUMBER"
	+ "\\pnrnot0\\pnf3\\pnstart1\\pnindent360\\pnhang{\\pntxtb \\'b7}} "
	+ "\n\\ls%LIST_NUMBER"
	;
private final static String LISTITEM_START_OL
	= "\n{\\*\\pn \\pnlvlbody\\ilvl0\\ls%LIST_NUMBER"
	+ "\\pnrnot0\\pndec\\pnstart1\\pnindent860\\pnhang{\\pntxta .}} "
	+ "\n\\ls%LIST_NUMBER"
	;
private final static String LISTITEM_START_AFTER
	= "\\adjustright "
	+ "\n{"
	;

private static final String LISTTABLEENTRY_START
	= "\n{\\list\\listtemplateid%LISTTEMPLATEID\\listsimple";
private static final String LISTTABLEENTRY_END
	= "\\fi-360\\li360\\jclisttab\\tx360 }"
	+ "\n{\\listname ;}\\listid%LISTID}" 
	;
private static final String LISTTABLEENTRY_UL
	= "\n{\\listlevel\\levelnfc23\\leveljc0\\levelfollow0\\levelstartat1\\levelspace0\\levelindent0"
	+ "\n{\\leveltext\\'01\\u-3913 ?;}{\\levelnumbers;}\\f3\\fbias0 "
	;
private static final String LISTTABLEENTRY_OL
	= "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelspace0\\levelindent0"
	+ "\n{\\leveltext\\'02\\'00.;}{\\levelnumbers\\'01;}"
	;
private static final String LISTTABLE_START
	= "\n{\\*\\listtable"
	+ "\n{\\list\\listtemplateid-1"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace144\\levelindent0"
	+ "\n{\\leveltext\\'01\\'00;}"
	+ "\n{\\levelnumbers\\'01;}\\s1}"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace144\\levelindent0"
	+ "\n{\\leveltext\\'03\\'00.\\'01;}"
	+ "\n{\\levelnumbers\\'01\\'03;}\\s2}"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace144\\levelindent0"
	+ "\n{\\leveltext\\'05\\'00.\\'01.\\'02;}"
	+ "\n{\\levelnumbers\\'01\\'03\\'05;}\\s3}"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace144\\levelindent0"
	+ "\n{\\leveltext\\'07\\'00.\\'01.\\'02.\\'03;}"
	+ "\n{\\levelnumbers\\'01\\'03\\'05\\'07;}\\s4}"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace144\\levelindent0"
	+ "\n{\\leveltext\\'09\\'00.\\'01.\\'02.\\'03.\\'04;}"
	+ "\n{\\levelnumbers\\'01\\'03\\'05\\'07\\'09;}\\s5}"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace144\\levelindent0"
	+ "\n{\\leveltext\\'0b\\'00.\\'01.\\'02.\\'03.\\'04.\\'05;}"
	+ "\n{\\levelnumbers\\'01\\'03\\'05\\'07\\'09\\'0b;}\\s6}"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace144\\levelindent0"
	+ "\n{\\leveltext\\'0d\\'00.\\'01.\\'02.\\'03.\\'04.\\'05.\\'06;}"
	+ "\n{\\levelnumbers\\'01\\'03\\'05\\'07\\'09\\'0b\\'0d;}\\s7}"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace144\\levelindent0"
	+ "\n{\\leveltext\\'0f\\'00.\\'01.\\'02.\\'03.\\'04.\\'05.\\'06.\\'07;}"
	+ "\n{\\levelnumbers\\'01\\'03\\'05\\'07\\'09\\'0b\\'0d\\'0f;}\\s8}"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace144\\levelindent0"
	+ "\n{\\leveltext\\'11\\'00.\\'01.\\'02.\\'03.\\'04.\\'05.\\'06.\\'07.\\'08;}"
	+ "\n{\\levelnumbers\\'01\\'03\\'05\\'07\\'09\\'0b\\'0d\\'0f\\'11;}\\s9}"
	+ "\n{\\listname ;}\\listid-5}"
	+ "\n{\\list\\listtemplateid-1\\listsimple"
	+ "\n{\\listlevel\\levelnfc0\\leveljc0\\levelfollow0\\levelstartat0\\levelspace0\\levelindent0"
	+ "\n{\\leveltext\\'01*;}"
	+ "\n{\\levelnumbers;}}"
	+ "\n{\\listname ;}\\listid-2}"
	;
	
private static final String LISTOVERRIDETABLE_ENTRY 
	= "\n{\\listoverride\\listid%LISTID\\listoverridecount0\\ls%LS}" 
	;
private static final String LISTOVERRIDETABLE_START
	= "\n{\\*\\listoverridetable"
	+ "\n{\\listoverride\\listid-5\\listoverridecount0\\ls1}"
	+ "\n{\\listoverride\\listid-2\\listoverridecount1"
	+ "\n{\\lfolevel\\listoverrideformat"
	+ "\n{\\listlevel\\levelnfc23\\leveljc0\\levelfollow0\\levelstartat1\\levelold\\levelspace0\\levelindent283"
	+ "\n{\\leveltext\\'01\\u-3913 ?;}"
	+ "\n{\\levelnumbers;}\\f3\\fbias0 \\fi-283\\li283 }}\\ls2}"
	;
	
private static final String STD_UNKNOWN
	= "\n\\fet0\\sectd"
	+ "\n\\linex0\\headery709\\footery709\\colsx709\\sectdefaultcl"
	+ "\n{\\*\\pnseclvl1\\pnucrm\\pnstart1\\pnindent720\\pnhang{\\pntxta .}}"
	+ "\n{\\*\\pnseclvl2\\pnucltr\\pnstart1\\pnindent720\\pnhang{\\pntxta .}}"
	+ "\n{\\*\\pnseclvl3\\pndec\\pnstart1\\pnindent720\\pnhang{\\pntxta .}}"
	+ "\n{\\*\\pnseclvl4\\pnlcltr\\pnstart1\\pnindent720\\pnhang{\\pntxta )}}"
	+ "\n{\\*\\pnseclvl5\\pndec\\pnstart1\\pnindent720\\pnhang{\\pntxtb (}{\\pntxta )}}"
	+ "\n{\\*\\pnseclvl6\\pnlcltr\\pnstart1\\pnindent720\\pnhang{\\pntxtb (}{\\pntxta )}}"
	+ "\n{\\*\\pnseclvl7\\pnlcrm\\pnstart1\\pnindent720\\pnhang{\\pntxtb (}{\\pntxta )}}"
	+ "\n{\\*\\pnseclvl8\\pnlcltr\\pnstart1\\pnindent720\\pnhang{\\pntxtb (}{\\pntxta )}}"
	+ "\n{\\*\\pnseclvl9\\pnlcrm\\pnstart1\\pnindent720\\pnhang{\\pntxtb (}{\\pntxta )}}"
	;
				
public static final String PARAGRAPH_BEFORE = "\n\\pard\\plain";
public static final String PARAGRAPH_END = "\\par}\n";
public static final String PARAGRAPH
  = "\n\\qj "
  + "\n\\sl300\\slmult1\\widctlpar\\aspalpha\\aspnum\\faauto\\adjustright "
  + "\n\\f1\\fs22\\lang1031\\langfe1031\\cgrid\\langnp1031\\langfenp1031"
  + "\n{";

public static final String TABLE_CELL_BORDER
	= "\n\\clbrdrt\\brdrth"
	+ "\n\\clbrdrl\\brdrth"
	+ "\n\\clbrdrb\\brdrth" 
	+ "\n\\clbrdrr\\brdrth"
	;
}