###################################################################################################
# zmscontainerobject.py
#
# Implementation of class ZMSContainerObject (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from OFS.Folder import Folder
from Globals import HTMLFile, MessageDialog, Persistent
import string 
import urllib
import time
# Product Imports.
from zmsobject import ZMSObject
from _metadata import Metadata
import _accessmanager
import _versionmanager
import _exportable
import _linklib
import _fileutil
import _globals
import _blobfields


###################################################################################################
###################################################################################################
###
###   A b s t r a c t   C l a s s
###
###################################################################################################
###################################################################################################
class ZMSContainerObject(
	ZMSObject, 
	Folder, 
	Metadata, 
	_accessmanager.AccessManager,		# Access manager.
	_versionmanager.VersionManagerContainer # Version manager.
	):

    # Management Permissions.
    # -----------------------
    __administratorPermissions__ = (
    		'manage_system',
                )
    __ac_permissions__=(
                ('ZMS Administrator', __administratorPermissions__),
		)

    # Interface.
    # ----------
    bodyContent_Teaser = HTMLFile('dtml/zmsteasercontainer/bodycontent_teaser', globals())
    bodyContent_ContextTeaser = HTMLFile('dtml/zmsteasercontainer/bodycontent_teaser', globals()) # DEPRECATED
    
    # Management Interface.
    # ---------------------
    zmscontainerobject_input_js = HTMLFile('dtml/zmscontainerobject/input_js', globals()) # JavaScript
    main_js = HTMLFile('dtml/zmscontainerobject/main_js', globals()) # JavaScript
    manage_main = HTMLFile('dtml/zmscontainerobject/manage_main', globals()) 
    manage_main_btn = HTMLFile('dtml/zmscontainerobject/manage_main_btn', globals()) # Buttons
    manage_main_change = HTMLFile('dtml/zmscontainerobject/manage_main_change', globals()) # Change (Author & Date)
    manage_main_actions = HTMLFile('dtml/zmscontainerobject/manage_main_actions', globals()) # Actions
    manage_search = HTMLFile('dtml/zmscontainerobject/manage_search', globals()) 
    manage_properties = HTMLFile('dtml/zmscontainerobject/manage_properties', globals())
    manage_system = HTMLFile('dtml/zmscontainerobject/manage_system', globals())

    # Sitemap.
    # --------
    sitemap_tree = HTMLFile('dtml/zmscontainerobject/sitemap/tree', globals()) 
    sitemap_params = HTMLFile('dtml/zmscontainerobject/sitemap/params', globals()) 
    sitemap_layout0 = HTMLFile('dtml/zmscontainerobject/sitemap/version0', globals()) 
    sitemap_layout1 = HTMLFile('dtml/zmscontainerobject/sitemap/version1', globals()) 
    sitemap_layout2 = HTMLFile('dtml/zmscontainerobject/sitemap/version2', globals())

    # Role Manager.
    # -------------    
    def manage_addZMSRubrik(self): pass
    def manage_addZMSDocument(self): pass
    def manage_addZMSTextarea(self): pass
    def manage_addZMSGraphic(self): pass
    def manage_addZMSFile(self): pass
    def manage_addZMSTable(self): pass
    def manage_addZMSNote(self): pass
    def manage_addZMSCustom(self): pass
    def manage_addZMSSysFolder(self): pass
    def manage_addZMSTeaserContainer(self): pass
    def manage_addZMSTeaserElement(self): pass
    def manage_addZMSLinkContainer(self): pass


    ###############################################################################################
    # ZMSContainerObject.recurse_updateVersion:
    #
    # Update version.
    ###############################################################################################
    def recurse_updateVersion(self, lang, manage_lang, REQUEST):
        message = ''
        
        # Process super-objects.
        message = message + ZMSObject.recurse_updateVersion(self,lang,manage_lang,REQUEST)
        message = message + self.metadatarecurse_updateVersion(lang,manage_lang,REQUEST)
        
        # Process object.
        # [...]
        
        # ---------------------------------------------------------------------
        # 11.01.2002 - 0.9 to 1.0: 
        # Blobs.
        # ---------------------------------------------------------------------
        _blobfields.synchronizeBlobFields(self)
        
        # ---------------------------------------------------------------------
        # 26.02.2002
        # Build #067: Activity-Toggle.
        # ---------------------------------------------------------------------
        if getattr(self.getDocumentElement(),'build','') < '067':
          for s_preview in [None,'preview']:
            req = {'preview':s_preview}
            ob = self.getObjVersion(req)
            for s_key in ['active']:
              for s_lang in self.getLanguages():
                attr = '%s_%s'%(s_key,s_lang)
                if not hasattr(ob,attr) or getattr(self,'build','') <= '067':
                  active = getattr(ob,'attr_active_start_%s'%s_lang,None) is not None or \
                           getattr(ob,'attr_active_end_%s'%s_lang,None) is not None 
                  #++ print "[%s.recurse_updateVersion]: Activity-Toggle %s.%s=%i"%(self.meta_type,str(ob),attr,active)
                  setattr(ob,attr,active)
        
        # Recurse through child-nodes.
        for ob in self.getChildNodes():
          message = message + ob.recurse_updateVersion(lang,manage_lang,REQUEST)
        
        # Return with message.
        return message


    ###############################################################################################
    #  ZMSContainerObject.manage_deleteObjs:
    #
    #  Delete a subordinate object:
    #  The objects specified in 'ids' get deleted.
    ###############################################################################################
    def manage_deleteObjs(self, lang, manage_lang, ids, REQUEST):
      """ ZMSContainerObject.manage_deleteObjs """ 
      
      message = ''
      
      ##### Delete objects ####
      count = 0
      for child in self.getChildNodes():
        if child.id in ids:
          if self.getAutocommit() or child.inObjStates(['STATE_NEW'],REQUEST):
            self.manage_delObjects([child.id],None)
            self.normalizeSortIds()
          else:
            child.setObjStateDeleted(REQUEST)
          count = count + 1
                  
      ##### VersionManager ####
      self.__autoCommit__(REQUEST)
      
      # Return with message.
      message = self.getLangStr('MSG_DELETED',manage_lang)%count
      if REQUEST.RESPONSE:
        return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))


    """
    ###############################################################################################    
    #
    #   Customize
    #
    ###############################################################################################    
    """

    # Customize Interfaces.
    
    def customize_manage_options(self, lang, manage_lang, REQUEST):
      options = []
      options.append({'label':'&lt;&lt; %s'%self.getLangStr('BACK',manage_lang),'action':'manage_main'})
      options.append({'label':self.getLangStr('TAB_CONFIGURATION',manage_lang),'action':'manage_customizeForm'})
      options.append({'label':self.getLangStr('TAB_LANGUAGES',manage_lang),'action':'manage_customizeLanguagesForm'})
      options.append({'label':self.getLangStr('TAB_METADATA',manage_lang),'action':'manage_customizeMetadictForm'})
      options.append({'label':self.getLangStr('TAB_METAOBJ',manage_lang),'action':'manage_customizeMetaobjForm'})
      options.append({'label':self.getLangStr('TAB_METAPATTERN',manage_lang),'action':'manage_customizeMetapatternForm'})
      options.append({'label':self.getLangStr('TAB_TEXTFORMAT',manage_lang),'action':'manage_customizeTextFormatForm'})
      options.append({'label':self.getLangStr('TAB_DESIGN',manage_lang),'action':'manage_customizeDesignForm'})
      return options


    """
    ###############################################################################################    
    #
    #   Properties
    #
    ###############################################################################################    
    """

    ###############################################################################################
    #  ZMSContainerObject.manage_changeProperties: 
    #
    #  Change properties.
    ###############################################################################################
    def manage_changeProperties(self, lang, manage_lang, REQUEST=None): 
        """ ZMSContainerObject.manage_changeProperties """
        
        if REQUEST.has_key('btn') and REQUEST['btn'] == self.getLangStr('BTN_CANCEL',manage_lang):
          return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s'%(lang,manage_lang))
        
        ##### Object State #####
        self.setObjStateModified(REQUEST)
            
        ##### Properties #####
        root = self.getDocumentElement().aq_parent
        # Alias (0)
        if REQUEST['AUTHENTICATED_USER'].has_permission('ZMS Administrator',self):
          dtml_id = string.lower(self.id_quote(self.getObjProperty('titleshort',REQUEST)))
          if dtml_id in root.objectIds(['DTML Method']):
            root.manage_delObjects([dtml_id],None)
        # Title.
        self.setReqProperty('title',REQUEST)
        self.setReqProperty('titleshort',REQUEST)
        self.setReqProperty('titleimage',REQUEST)
        # Active.
        self.setReqProperty('active',REQUEST)
        self.setReqProperty('attr_active_start',REQUEST)
        self.setReqProperty('attr_active_end',REQUEST)
        # Levelnfc
        if 'levelnfc' in self.getObjAttrs().keys():
          self.setReqProperty('levelnfc',REQUEST)
        # Cacheable
        if REQUEST['AUTHENTICATED_USER'].has_permission('ZMS Administrator',self) and \
           self.isCacheActive() and 'attr_cacheable' in self.getObjAttrs().keys():
          self.setReqProperty('attr_cacheable',REQUEST)
        # Alias (1)
        if REQUEST['AUTHENTICATED_USER'].has_permission('ZMS Administrator',self):
          if 'attr_alias' in self.getObjAttrs().keys():
            self.setReqProperty('attr_alias',REQUEST)
            dtml_id = string.lower(self.id_quote(self.getObjProperty('titleshort',REQUEST)))
            if self.getObjProperty('attr_alias',REQUEST):
              dtml_title = "Redirect to %s"%self.getObjProperty('titleshort',REQUEST)
              path = 'content/%s/getHref2IndexHtml(REQUEST)'%self.relative_obj_path()
              path = string.replace(path,'/','.')
              dtml_raw = ""
              dtml_raw = dtml_raw + '<!-- BO %s -->\n'%dtml_id
              dtml_raw = dtml_raw + '  <dtml-call "RESPONSE.redirect(%s)">\n'%path
              dtml_raw = dtml_raw + '<!-- EO %s -->\n'%dtml_id
              root.manage_addDTMLMethod(dtml_id,dtml_title,dtml_raw)
        
        ##### Blobs #####
        _blobfields.synchronizeBlobFields(self)
        
        ##### Metadata #####
        self.setMetadata(lang,manage_lang,REQUEST)
        
        ##### VersionManager ####
        self.__autoCommit__(REQUEST)
        
        # Return with message.
        message = self.getLangStr('MSG_CHANGED',manage_lang)
        if REQUEST and hasattr(REQUEST,'RESPONSE'):
          if REQUEST.RESPONSE:
            return REQUEST.RESPONSE.redirect('manage_properties?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))

    # -------------------------------------------------------------------------------------------------
    #	ZMSContainerObject.isPage:
    # -------------------------------------------------------------------------------------------------
    def isPage(self): return 1

    # ---------------------------------------------------------------------------------------------
    #	ZMSContainerObject.isResource
    # ---------------------------------------------------------------------------------------------
    def isResource(self,REQUEST): 
      rtn = 0
      dc_type = self.getObjProperty('attr_dc_type',REQUEST)
      rtn = rtn or (type(dc_type) is type('') and dc_type == 'Resource')
      rtn = rtn or (type(dc_type) is type([]) and dc_type == ['Resource'])
      return rtn

    # ---------------------------------------------------------------------------------------------
    #  	ZMSContainerObject.isActive
    # ---------------------------------------------------------------------------------------------
    def isActive(self,REQUEST):
      rtn = self.getObjProperty('active',REQUEST)
      if rtn:
        #-- Init time.
        t0 = time.time()
        #-- Start.  
        t1 = self.getObjProperty('attr_active_start',REQUEST)
        if t1 is not None:
          rtn = rtn and _globals.compareDate(t1,t0) >= 0
        #-- End.  
        t1 = self.getObjProperty('attr_active_end',REQUEST)
        if t1 is not None:
          rtn = rtn and _globals.compareDate(t1,t0) < 1
      return rtn

    # ---------------------------------------------------------------------------------------------
    #  ZMSContainerObject.getLevelnfc
    # ---------------------------------------------------------------------------------------------
    def _getLevelnfc(self, REQUEST):
      s = ''
      if self.getLevel() > 0:
        parent = self.getParentNode()
        try:
          s = parent._getLevelnfc(REQUEST)
          levelnfc = parent.getObjProperty('levelnfc',REQUEST)
          if len(levelnfc) > 0:
            siblings = parent.filteredChildNodes(REQUEST,self.PAGES)
            if self in siblings:
              i = siblings.index(self)
              if levelnfc == '0':
                s = s + str(i + 1) + '.'        
              elif levelnfc == '1':
                s = s + chr(i + ord('A')) + '.'
              elif levelnfc == '2':
                s = s + chr(i + ord('a')) + '.'
        except:
          pass
      return s

    def getLevelnfc(self, REQUEST):
      s = ''
      if self.getLevel() > 0:
        parent = self.getParentNode()
        try:
          levelnfc = parent.getObjProperty('levelnfc',REQUEST)
          if len(levelnfc) > 0:
            s = self._getLevelnfc(REQUEST)
            if len(s) > 0: s = s + ' '
        except:
          pass
      return s

    # ---------------------------------------------------------------------------------------------
    #  ZMSContainerObject.getTitle
    # ---------------------------------------------------------------------------------------------
    def getTitle(self, REQUEST): return self.getLevelnfc(REQUEST) + self.getObjProperty('title',REQUEST) 

    # ---------------------------------------------------------------------------------------------
    #  ZMSContainerObject.getTitlealt
    # ---------------------------------------------------------------------------------------------
    def getTitlealt(self, REQUEST): return self.getLevelnfc(REQUEST) + self.getObjProperty('titleshort',REQUEST) 

    # ---------------------------------------------------------------------------------------------
    #  ZMSContainerObject.getTitleimage
    # ---------------------------------------------------------------------------------------------
    def getTitleimage(self, REQUEST): return self.getObjProperty('titleimage',REQUEST) 

    # ---------------------------------------------------------------------------------------------
    #  ZMSContainerObject.getContentType
    # ---------------------------------------------------------------------------------------------
    def getContentType(self, REQUEST): return 'text/html'

    # ---------------------------------------------------------------------------------------------
    #  ZMSContainerObject.getDCCoverage
    # ---------------------------------------------------------------------------------------------
    def getDCCoverage(self, REQUEST): return self.getObjProperty('attr_dc_coverage',REQUEST)


    """
    ###############################################################################################    
    #
    #   I M P O R T  /  E X P O R T
    #
    ###############################################################################################    
    """

    # Interface.
    manage_importexport = HTMLFile('dtml/zmscontainerobject/manage_importexport', globals()) 

    ###############################################################################################    
    #  ZMSContainerObject.manage_import:
    #
    #
    ###############################################################################################    
    def manage_import(self, file, lang, manage_lang, REQUEST):
        """ ZMSContainerObject.manage_import """
        from cStringIO import StringIO
        import os
        import tempfile
        
        message = ''
        
        # Create temporary folder.
        sTmpFolder = tempfile.mktemp()
        os.mkdir(sTmpFolder)
        
        # Save ZIP archive to temporary file.
        sTmpZipFilename = '%s/%s'%(sTmpFolder,_fileutil.extractFileName(file.filename))
        fTmpZip = open( sTmpZipFilename,'wb')
        fTmpZip.write( file.read() )
        fTmpZip.close()
            
        # Extract ZIP archive.
        _fileutil.extractZipArchive(sTmpZipFilename)
        
        # Search for XML file in ZIP archive.
        # -----------------------------------
        sTmpXmlFilename = _fileutil.findExtension('xml',sTmpFolder)
        if sTmpXmlFilename is None:
          # Return message.
          message = 'XML-File not found!'
        else:
        
          # Read content of XML-file.
          f = open(sTmpXmlFilename,'r')
          xml = f.read()
          f.close()
          
          # Process root.
          bInRootTag = self.id == self.getDocumentElement().id
          if bInRootTag:
            # remove all ZMS-objects.
            self.manage_delObjects(self.objectIds([self.dGlobalAttrs.keys()]),None)
            # remove all languages.
            for s_lang in self.getLanguages():
              self.__del_lang__(s_lang)
          else:
            self.dTagStack = _globals.MyStack()
            self.dValueStack = _globals.MyStack()
          
          self.oParent = self.getParentNode()
          
          # parse XML-file.
          self.parse(StringIO(xml),self,1)
          
          # remove helpers after import
          self.recurse_afterImport()
          
          # upload resource files.
          sTmpZipFolder = _fileutil.getFilePath(sTmpXmlFilename)
          message = message + self.recurse_uploadRessources(sTmpZipFolder)
          
          ##### VersionManager ####
          self.__autoCommit__(REQUEST)
          
          # Return message.
          message = message + self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%_fileutil.extractFileName(file.filename))
        
        # Remove temporary files.
        _fileutil.rmFilePath(sTmpFolder)
        
        # return with message
        message = urllib.quote(message)
        return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    """
    ###############################################################################################
    ###  
    ###  Related Links 
    ### 
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMSContainerObject.getLinkList:
    #
    #	Returns list of link-URLs for child-objects.
    # ---------------------------------------------------------------------------------------------
    def getLinkList(self, REQUEST, allow_none=0):
      rtn = []
      for ob in self.getChildNodes(REQUEST,['ZMSFile','ZMSLinkContainer']):
        rtn.extend(ob.getLinkList(REQUEST,allow_none))
      return rtn


    """
    ###############################################################################################    
    ###  
    ###  Auswahlliste fr Aktionen 
    ### 
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMSContainerObject.filtered_insert_actions:
    #
    #	Object-actions of management interface.
    # ---------------------------------------------------------------------------------------------
    def filtered_insert_actions(self, path, REQUEST):
        rtn = []
    
    	lang = REQUEST['lang']
    	manage_lang = REQUEST['manage_lang']
    	
    	# Authenticated User.
        auth_user = REQUEST['AUTHENTICATED_USER']
        permissions = self.getAuthUserPermissions(REQUEST)
        
        # ZMS-Objects.
        # ------------
        actions = []
        for ac_inherited_permission in self.ac_inherited_permissions(1):
          ac_permission = ac_inherited_permission[0]
          if ac_permission in permissions:
            ac_actions = ac_inherited_permission[1]
            for ac_action in list(ac_actions):
              if string.find(ac_action,'manage_addZMS') == 0:
                meta_type = ac_action[10:]
                if self.getAutocommit() or \
                   self.getPrimaryLanguage() == lang or \
                   self.getDCCoverage(REQUEST) not in ['obligation','proposal'] or \
                   meta_type in ['ZMSRubrik','ZMSDocument']:
                  if meta_type != 'ZMSSysFolder' or auth_user.has_permission('ZMS Administrator',self):
                    constructor = self.dGlobalAttrs[meta_type]['constructor']
                    if constructor is not None:
                      record = (self.display_type(REQUEST,meta_type),'manage_addProduct/zms/%s'%self.dGlobalAttrs[meta_type]['constructor'],[])
                      if not record in actions:
                        actions.append(record)
        # Sort.
        actions.sort()
        
        # Meta-Objects.
        # -------------
        meta_objs = []
        if self.getAutocommit() or \
           self.getPrimaryLanguage() == lang or \
           self.getDCCoverage(REQUEST) not in ['obligation','proposal']:
          for id in self.getMetaobjIds(REQUEST):
            ob = self.getMetaobj(id)
            if (self.meta_type in ['ZMS','ZMSRubrik'] and ob['type'] in ['ZMSDocument','ZMSObject']) or \
               (self.meta_type in ['ZMSDocument'] and ob['type'] in ['ZMSObject']) or \
               (self.meta_type in ['ZMSTeaserContainer'] and ob['type'] in ['ZMSTeaserElement']):
              record = (ob['name'],'manage_addProduct/zms/manage_addzmscustomform',[])
              if not record in meta_objs:
                meta_objs.append(record)
        # Sort.
        meta_objs.sort()
        # Heading.
        if len(meta_objs) > 0:
          actions.append((self.getLangStr('CAPTION_INSERT',manage_lang)%self.getLangStr('TAB_METAOBJ',manage_lang),'',meta_objs))

        # Meta-Patterns.
        # --------------
        meta_patterns = []
        if self.getAutocommit() or \
           self.getPrimaryLanguage() == lang or \
           self.getDCCoverage(REQUEST) not in ['obligation','proposal']:
          for id in self.getMetapatternIds(REQUEST):
            ob = self.getMetapattern(id,REQUEST)
            record = (ob['title'],'manage_instantiateMetapattern',[])
            if not record in meta_patterns:
              meta_patterns.append(record)
        # Sort.
        meta_patterns.sort()
        # Add.
        record = (self.getLangStr('ACTION_INSERT_METAPATTERN',manage_lang),path + 'manage_addMetapatternForm',[])
        if ((self.getLevel()) > 0 or (len(path) > 0 and hasattr(self,path[:-1]) and getattr(self,path[:-1]).isPage())) and \
           self.isPage() and \
           auth_user.has_permission('ZMS Administrator',self):
          meta_patterns.append(record)
        # Heading.
        if len(meta_patterns) > 0:
          actions.append((self.getLangStr('CAPTION_INSERT',manage_lang)%self.getLangStr('TAB_METAPATTERN',manage_lang),'',meta_patterns))
        
        # Heading
        if len(actions) > 0:
          rtn.append((self.getLangStr('ACTION_INSERT',manage_lang)%self.display_type(REQUEST),'',actions))
        
        # Return insert-action list.
        return rtn


    # ---------------------------------------------------------------------------------------------
    #	ZMSContainerObject.filtered_container_actions:
    #
    #	Object-actions of management interface.
    # ---------------------------------------------------------------------------------------------
    def filtered_container_actions(self, REQUEST):
      rtn = []
      
      lang = REQUEST['lang']
      manage_lang = REQUEST['manage_lang']
      auth_user = REQUEST['AUTHENTICATED_USER']
      
      #-- WORKFLOW
      if self.getContainerObjState(REQUEST):
        actions = []
        if auth_user.has_permission('Manager',self) or \
           auth_user.has_permission('ZMS Administrator',self) or \
           auth_user.has_permission('ZMS Editor',self):
          actions.append((self.getLangStr('BTN_COMMIT',manage_lang),'manage_commitVersionChanges',[]))
          if self.inObjStates(['STATE_REQUESTED'],REQUEST):
            actions.append((self.getLangStr('BTN_REJECT',manage_lang),'manage_rejectVersionChanges',[]))
        if auth_user.has_permission('ZMS Author',self):
          actions.append((self.getLangStr('BTN_ROLLBACK',manage_lang),'manage_rollbackVersionChanges',[]))
          if not self.inObjStates(['STATE_REQUESTED'],REQUEST):
            actions.append((self.getLangStr('BTN_REQUESTCOMMIT',manage_lang),'manage_requestVersionChanges',[]))
        if len(actions) > 0:
          rtn.append((self.getLangStr('ATTR_CHANGES',manage_lang),'',actions))

      #-- INSERT
      rtn.extend(self.filtered_insert_actions('',REQUEST))
      
      # Return action list.
      return rtn


    """
    ###############################################################################################
    ###
    ###  HTML-Darstellung 
    ###
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMSContainerObject.hasPageElements:
    # ---------------------------------------------------------------------------------------------
    def hasPageElements(self, REQUEST):
      for ob in self.getChildNodes(REQUEST):
        if ob.isVisible(REQUEST): 
          if ob.isPageElement():
            return 1
      return 0

    # ---------------------------------------------------------------------------------------------
    #  ZMSContainerObject.getPageSiblingMapping:
    #
    #  The page immediately preceding this page, otherwise returns None. 
    # ---------------------------------------------------------------------------------------------
    def getPageSiblingMapping(self, obs, REQUEST, management_interface=0):
      dct = {}
      for ob in obs:
        dct[ob.id] = ob.getPageSibling(REQUEST,management_interface)
      return dct

    # ---------------------------------------------------------------------------------------------
    #  ZMSContainerObject.getPageSibling: 
    #
    #  The page immediately preceding this page, otherwise returns None. 
    # ---------------------------------------------------------------------------------------------
    def getPageSibling(self, REQUEST=None, management_interface=0):
      parent = self.getParentNode()
      if parent is not None:
        siblings = parent.getChildNodes(REQUEST,self.PAGES) 
        if self in siblings:
          i = siblings.index(self) + 1
          for k in range(i,len(siblings)):
            ob = siblings[k]
            if management_interface or (ob.isVisible(REQUEST) and not ob.isResource(REQUEST)):
              return ob
      return None


    ###############################################################################################
    # ZMSContainerObject.getSitemapElements: 
    #
    # Elemente der Sitemap
    ###############################################################################################
    def getSitemapElements(self, REQUEST, expand_default=2, management_interface=0, current=None):
        
        if current is None:
          
          return self.getDocumentElement().getSitemapElements(REQUEST,expand_default,management_interface,self)
        
        else:
          
          elmnts = []
          
          append = 1
          append = append and (self.isPage())
          append = append and (management_interface or self.isVisible(REQUEST)) # Object is visible.
          append = append and (management_interface or not self.isResource(REQUEST)) # Object is not resource.
          if append: 
            elmnts.append(self)
            
            # Child navigation.
            if self.getLevel() < expand_default or self.isAnchestor(current):
              for ob in self.getChildNodes(REQUEST,self.PAGES):
                elmnts.extend(ob.getSitemapElements(REQUEST,expand_default,management_interface,current))
          
          # Return elements.
          return elmnts


    ###############################################################################################
    # ZMSContainerObject.getNavElements: 
    #
    # Elemente der Hauptnavigation im Contentbereich
    ###############################################################################################
    def getNavElements(self, REQUEST, current_child=None, subElements=[]):
    
        elmnts = []
        
        # Child navigation.
        if current_child is not None and \
           current_child.meta_type not in ['ZMSDocument','ZMSCustom'] and \
           self.hasPageElements(REQUEST) and \
           self.getLevel() > 0:
          obs = [current_child]
        else:     
          obs = self.getChildNodes(REQUEST,self.PAGES)
        
        for ob in obs:
          append = 1
          append = append and (ob.isPage())
          append = append and (ob.isVisible(REQUEST)) # Object is visible.
          append = append and (not ob.isResource(REQUEST)) # Object is not resource.
          if append: elmnts.append(ob)
          
          if current_child is not None and \
             ob.id == current_child.id and \
             ob.hasAccess(REQUEST):
            elmnts.extend(subElements)
         
        # Parent navigation.
        parent = self.getParentNode()
        if parent is not None:
          elmnts = parent.getNavElements(REQUEST,self,elmnts)
        
        # Return elements.
        return elmnts


    ###############################################################################################
    # ZMSContainerObject.getIndexNavElements: 
    #
    # Elemente der Index-Navigation im Contentbereich
    ###############################################################################################
    def getIndexNavElements(self, REQUEST):
      
      elmnts = []
      el_meta_types = ['ZMSDocument','ZMSCustom']
      
      # Child navigation.
      for ob in self.getChildNodes(REQUEST,el_meta_types):
        append = 1
        append = append and (not REQUEST.has_key('op') or not REQUEST['op'] in ['sitemap','search']) # No operation (Sitemap / Search)
        append = append and (ob.isPage())
        append = append and (ob.isVisible(REQUEST)) # Object is visible.
        append = append and (not ob.isResource(REQUEST)) # Object is not resource.
        if append:
          elmnts.append(ob)
      
      # Return elements.
      return elmnts

    ###############################################################################################    
    # ZMSContainerObject.renderShort:
    #
    # Renders short presentation of a ContainerObject.
    ###############################################################################################    
    def renderShort(self,REQUEST):
      rtn = string.strip(self.getTitlealt(REQUEST))
      if len(rtn) == 0:
        rtn = '<i>&lt;%s&gt;</i>'%(self.display_type(REQUEST))
      rtn = '<h2>%s</h2>'%rtn
      return rtn

    ###############################################################################################    
    # ZMSContainerObject.printHtml:
    #
    # Renders print presentation of a ContainerObject.
    ###############################################################################################    
    def printHtml(self, level, sectionize, sectionizer, REQUEST):
      html = ""
      
      # Title.
      title = self.getObjProperty("title",REQUEST)
      if level == 0:
        html = html + '<p><div align="center"><center><font style=\"font-size:20px; font-weight:bold;\">%s</font></center></div><br></p>'%title
      else:
        sectionizer.processLevel(level)
        if sectionize:
          title = '%s %s'%(str(sectionizer),title)
        html = html + "<h%i>%s</h%i>"%(level,title,level)

      # Creator.
      for key in ['attr_creator','attr_dc_creator']:
        if key in self.getObjAttrs().keys():
          val = self.getObjProperty(key,REQUEST)
          if len(val) > 0:
            html = html + '<div align="center"><center>%s %s</center></div>'%(self.getLangStr('BY',self.getManageLanguage(REQUEST['lang'])),val)
      
      # Description.
      for key in ['attr_description','attr_dc_description']:
        if key in self.getObjAttrs().keys():
          val = self.getObjProperty(key,REQUEST)
          if len(val) > 0:
            html = html + '<em>%s</em>'%val
      
      # Objects.
      subsectionizer = {}
      for ob in self.filteredChildNodes(REQUEST,self.PAGEELEMENTS):
        if not subsectionizer.has_key(ob.meta_type):
          subsectionizer[ob.meta_type] = sectionizer.clone()
        subsectionizer[ob.meta_type].processLevel(level+1)
        html = html + ob.printHtml(level+1,sectionize,subsectionizer[ob.meta_type],REQUEST)
        
      # Container-Objects.      
      for ob in self.filteredChildNodes(REQUEST,self.PAGES):
        html = html + ob.printHtml(level+1,sectionize,sectionizer,REQUEST)
        
      return html

    ###############################################################################################    
    #	ZMSContainerObject.catalogText:
    #
    #	Catalog text.
    ###############################################################################################    
    def catalogText(self, REQUEST):
      s = ""
      s = s + self.getTitle(REQUEST) + " "
      s = s + self.getTitlealt(REQUEST) + " " 
      for ob in self.filteredChildNodes(REQUEST,self.PAGEELEMENTS):
        s = s + ob.catalogText(REQUEST)
      return s


    """
    ###############################################################################################    
    ###  
    ###  DOM-Methoden 
    ### 
    ###############################################################################################    
    """

    # Constants.
    # ----------
    PAGES = 0
    PAGEELEMENTS = 1

    # ---------------------------------------------------------------------------------------------
    #	ZMSContainerObject.filteredTreeNodes:
    #
    #	Returns a NodeList that contains all accessible children of this tree in correct order.
    #	If none, this is a empty NodeList. 
    # ---------------------------------------------------------------------------------------------
    def filteredTreeNodes(self, REQUEST, meta_types, order_by=None, order_dir="asc", max_len=None, visible=1):
      rtn = []
      
      #-- RECURSE TREE
      obs = self.getChildNodes(REQUEST)
      for ob in obs:
        append = 1
        append = append and (type(meta_types) is not type('') or (ob.meta_type == 'ZMSCustom' and ob.meta_id == meta_types))
        append = append and (type(meta_types) is not type([]) or ob.meta_type in meta_types)
        append = append and (type(meta_types) is not type(self.PAGES) or meta_types != SELF.PAGES or ob.isPage())
        append = append and (type(meta_types) is not type(self.PAGEELEMENTS) or meta_types != SELF.PAGEELEMENTS or ob.isPageElement())
        if append: rtn.append(ob)
        if ob.isPage():
          rtn.extend(ob.filteredTreeNodes(REQUEST,meta_types,None,None,None,0))
      
      #-- VISIBLE
      if visible:
      
        # collect object-items
        tmp = []
        for ob in rtn:
          if ob.isVisible(REQUEST):
            tmp.append(ob)
        rtn = tmp
      
      #-- ORDER_BY
      if order_by is not None:
      
        # collect object-items
        tmp = []
        for ob in rtn:
          tmp.append((ob.getObjProperty(order_by,REQUEST),ob))
        
        # sort object-items
        tmp.sort()
        
        # truncate sort-id from sorted object-items
        rtn = []
        for ob in tmp:
          if order_dir == "asc":
            rtn.append(ob[1])
          else:
            rtn.insert(0,ob[1])
      
      #-- MAX_LEN
      if max_len is not None:
        if len(rtn) > max_len:
          rtn = rtn[:max_len]
      
      return rtn

    # ---------------------------------------------------------------------------------------------
    #	ZMSContainerObject.firstFilteredChildNode:
    #
    #	Returns the first accessible child of this node.
    # ---------------------------------------------------------------------------------------------
    def firstFilteredChildNode(self, REQUEST=None, meta_types=None, management_interface=0):
      obs = self.getChildNodes(REQUEST,meta_types)
      for ob in obs:
        if management_interface or ob.isVisible(REQUEST):
          return ob
      return None

    # ---------------------------------------------------------------------------------------------
    #	ZMSContainerObject.filteredChildNodes:
    #
    #	Returns a NodeList that contains all accessible children of this node in correct order.
    #	If none, this is a empty NodeList. 
    # ---------------------------------------------------------------------------------------------
    def filteredChildNodes(self, REQUEST=None, meta_types=None, management_interface=0):
      rtn = []
      obs = self.getChildNodes(REQUEST,meta_types)
      for ob in obs:
        if management_interface or ob.isVisible(REQUEST):
          rtn.append(ob)
      return rtn

    # ---------------------------------------------------------------------------------------------
    #	ZMSContainerObject.getChildNodes:
    #
    #	Returns a NodeList that contains all children of this node in correct sort-order.
    #	If none, this is a empty NodeList. 
    # ---------------------------------------------------------------------------------------------
    def getChildNodes(self, REQUEST=None, meta_types=None):
      
      # META_ID
      meta_id = None
      if meta_types is not None and type(meta_types) is type(''):
        meta_id = meta_types
        meta_types = ['ZMSCustom']
        
      # OBJ_TYPE
      obj_type = None
      if meta_types is not None and type(meta_types) is not type([]):
        obj_type = meta_types
        meta_types = None  
      if meta_types is None or type(meta_types) is not type([]):
        meta_types = self.dGlobalAttrs.keys()
      
      # collect object-items
      tmp = []
      obs = self.objectValues(meta_types)
      for ob in obs:
        append = 1
        append = append and (meta_id is None or ob.meta_id == meta_id)
        append = append and (obj_type != self.PAGES or ob.isPage())
        append = append and (obj_type != self.PAGEELEMENTS or ob.isPageElement())
        if append: 
          sort_id = ob.getObjVersion(REQUEST).sort_id
          if ob.isPage():
            sort_id = 's' + sort_id
          tmp.append((sort_id,ob))
      
      # sort object-items
      tmp.sort()
      
      # truncate sort-id from sorted object-items
      rtn = []
      for ob in tmp:
        rtn.append(ob[1])
      
      # return sorted object-items
      return rtn


    """
    ###############################################################################################    
    ###  
    ###  Sortier-Reihenfolge
    ### 
    ###############################################################################################    
    """

    ###############################################################################################
    #  ZMSContainerObject.normalizeSortIds:
    #
    #  Normalizes sort_ids of a ContainerObjects elements.
    ###############################################################################################    
    def normalizeSortIds(self):
      
      req = {}
      req['lang'] = self.getPrimaryLanguage()
      req['preview'] = 'preview'
      
      sort_id = 10
      for ob in self.getChildNodes(req):
       ob.setObjStateModifiedSilent(req)
       ob.setObjProperty('sort_id',_globals.format_sort_id(sort_id))
       sort_id = sort_id + 10
      
      self.__autoCommitOnlyModifiedSilent__()


    ###############################################################################################    
    #  ZMSContainerObject.getNewSortId:
    #
    #  Get new Sort-ID.
    ###############################################################################################    
    def getNewSortId(self):
      
      req = {}
      req['lang'] = self.getPrimaryLanguage()
      req['preview'] = 'preview'
      
      newSortId = 0
      for obj in self.getChildNodes(req):
        objSortId = obj.getSortId(req)
        if objSortId > newSortId:
          newSortId = objSortId
      newSortId = newSortId + 10
      
      return newSortId


    """
    ###############################################################################################    
    #
    #   T E A S E R 
    #
    ###############################################################################################    
    """

    ###############################################################################################
    #	ZMSContainerObject.getTeaserElements:
    #
    #	Returns Teaser-Objects.
    ###############################################################################################
    def getTeaserElements(self, REQUEST, current=None):
      rtn = []
      if current is None: current = self
      if current.hasAccess(REQUEST):
        # Process parent.
        if self.getParentNode() is not None:
          rtn = self.getParentNode().getTeaserElements(REQUEST,current)
        # Process this.
        for oTeaserContainer in self.filteredChildNodes(REQUEST,['ZMSTeaserContainer']):
          for oTeaserElement in oTeaserContainer.filteredChildNodes(REQUEST,['ZMSTeaserElement','ZMSCustom']):
            append = 0
            if (oTeaserElement.meta_type == 'ZMSTeaserElement'):
              # 0= this 
              append = append or \
                (current == self and \
                 oTeaserElement.getObjProperty('attr_penetrance',REQUEST) == 0)
              # 1= sub_nav
              append = append or \
                (current.meta_type in ['ZMS','ZMSRubrik'] and \
                 oTeaserElement.getObjProperty('attr_penetrance',REQUEST) == 1)
              # 2= sub_all
              append = append or \
                (oTeaserElement.getObjProperty('attr_penetrance',REQUEST) == 2)
            elif (oTeaserElement.meta_type == 'ZMSCustom'):
              append = append or \
                (current == self)
            #-- Append
            if append: rtn.append(oTeaserElement)
      # Return list of Teaser-Objects.
      return rtn


    """
    ###############################################################################################    
    #
    #   B U I L D E R
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMSContainerObject.recurse_afterImport:
    #
    # Remove helpers after XML-Import.
    ###############################################################################################
    def recurse_afterImport(self):
        self._recurse_afterImport()
        # Process children.
        for ob in self.getChildNodes():
          ob.recurse_afterImport()

###################################################################################################
